@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA.
  */
 
 #ifndef __MNOTE_OLYMPUS_TAG_H__
@@ -39,16 +39,38 @@ enum _MnoteOlympusTag {
 	MNOTE_NIKON_TAG_FLASHMODE               = 0x0009,
 	MNOTE_NIKON_TAG_WHITEBALANCEFINE        = 0x000b,
 	MNOTE_NIKON_TAG_WHITEBALANCERB          = 0x000c,
-	MNOTE_NIKON_TAG_UNKNOWN_0X000D          = 0x000d,
+	MNOTE_NIKON_TAG_PROGRAMSHIFT            = 0x000d,
 	MNOTE_NIKON_TAG_EXPOSUREDIFF            = 0x000e,
 	MNOTE_NIKON_TAG_ISOSELECTION            = 0x000f,
-	MNOTE_NIKON_TAG_PREVIEWIMAGE            = 0x0011,
+	MNOTE_NIKON_TAG_PREVIEWIMAGE_IFD_POINTER= 0x0011,
 	MNOTE_NIKON_TAG_FLASHEXPCOMPENSATION    = 0x0012,
 	MNOTE_NIKON_TAG_ISO2                    = 0x0013,
 	MNOTE_NIKON_TAG_IMAGEBOUNDARY           = 0x0016,
-	MNOTE_NIKON_TAG_UNKNOWN_0X0017          = 0x0017,
+	MNOTE_NIKON_TAG_EXTERNALFLASHEXPCOMPENSATION = 0x0017,
 	MNOTE_NIKON_TAG_FLASHEXPOSUREBRACKETVAL = 0x0018,
 	MNOTE_NIKON_TAG_EXPOSUREBRACKETVAL      = 0x0019,
+	MNOTE_NIKON_TAG_CROPHISPEED      	= 0x001b,
+	MNOTE_NIKON_TAG_EXPOSURETUNING      	= 0x001c,
+	MNOTE_NIKON_TAG_SERIALNUMBER      	= 0x001d,
+	MNOTE_NIKON_TAG_COLORSPACE      	= 0x001e,
+	MNOTE_NIKON_TAG_VRINFO      		= 0x001f,
+	MNOTE_NIKON_TAG_IMAGEAUTHENTICATION 	= 0x0020,
+	MNOTE_NIKON_TAG_FACEDETECT 		= 0x0021,
+	MNOTE_NIKON_TAG_ACTIVEDLIGHTING 	= 0x0022,
+	MNOTE_NIKON_TAG_PICTURECONTROLDATA 	= 0x0023,
+	MNOTE_NIKON_TAG_WORLDTIME 		= 0x0024,
+	MNOTE_NIKON_TAG_ISOINFO 		= 0x0025,
+	MNOTE_NIKON_TAG_VIGNETTECONTROL 	= 0x002a,
+	MNOTE_NIKON_TAG_DISTORTINFO 		= 0x002b,
+	MNOTE_NIKON_TAG_SHUTTERMODE 		= 0x0034,
+	MNOTE_NIKON_TAG_HDRINFO 		= 0x0035,
+	MNOTE_NIKON_TAG_MECHANICALSHUTTERCOUNT 	= 0x0037,
+	MNOTE_NIKON_TAG_LOCATIONINFO 		= 0x0039,
+	MNOTE_NIKON_TAG_BLACKLEVEL 		= 0x003d,
+	MNOTE_NIKON_TAG_IMAGESIZERAW 		= 0x003e,
+	MNOTE_NIKON_TAG_CROPAREA 		= 0x0045,
+	MNOTE_NIKON_TAG_NIKONSETTINGS 		= 0x004e,
+	MNOTE_NIKON_TAG_COLORTEMPERATUREAUTO 	= 0x004f,
 	MNOTE_NIKON_TAG_IMAGEADJUSTMENT         = 0x0080,
 	MNOTE_NIKON_TAG_TONECOMPENSATION        = 0x0081,
 	MNOTE_NIKON_TAG_ADAPTER                 = 0x0082,
@@ -58,28 +80,36 @@ enum _MnoteOlympusTag {
 	MNOTE_NIKON_TAG_DIGITALZOOM             = 0x0086,
 	MNOTE_NIKON_TAG_FLASHUSED               = 0x0087,
 	MNOTE_NIKON_TAG_AFFOCUSPOSITION         = 0x0088,
-	MNOTE_NIKON_TAG_BRACKETING              = 0x0089,
+	MNOTE_NIKON_TAG_BRACKETING              = 0x0089,	/* shootingmode? */
 	MNOTE_NIKON_TAG_UNKNOWN_0X008A          = 0x008a,
 	MNOTE_NIKON_TAG_LENS_FSTOPS             = 0x008b,
 	MNOTE_NIKON_TAG_CURVE                   = 0x008c,
 	MNOTE_NIKON_TAG_COLORMODE               = 0x008d,
 	MNOTE_NIKON_TAG_LIGHTTYPE               = 0x0090,
-	MNOTE_NIKON_TAG_UNKNOWN_0X0091          = 0x0091,
+	MNOTE_NIKON_TAG_SHOTINFO                = 0x0091,
 	MNOTE_NIKON_TAG_HUE                     = 0x0092,
 	MNOTE_NIKON_TAG_SATURATION              = 0x0094,
 	MNOTE_NIKON_TAG_NOISEREDUCTION          = 0x0095,
-	MNOTE_NIKON_TAG_UNKNOWN_0X0097          = 0x0097,
-	MNOTE_NIKON_TAG_UNKNOWN_0X0098          = 0x0098,
+	MNOTE_NIKON_TAG_COLORBALANCE            = 0x0097,
+	MNOTE_NIKON_TAG_LENSDATA          	= 0x0098,
 	MNOTE_NIKON_TAG_SENSORPIXELSIZE         = 0x009a,
 	MNOTE_NIKON_TAG_UNKNOWN_0X009B          = 0x009b,
-	MNOTE_NIKON_TAG_SERIALNUMBER            = 0x00a0,
+	MNOTE_NIKON_TAG_RETOUCHHISTORY          = 0x009e,
+	MNOTE_NIKON_TAG_SERIALNUMBER2		= 0x00a0,
 	MNOTE_NIKON_TAG_IMAGE_DATASIZE          = 0x00a2,
 	MNOTE_NIKON_TAG_UNKNOWN_0X00A3          = 0x00a3,
-	MNOTE_NIKON_TAG_TOTALPICTURES           = 0x00a7,
-	MNOTE_NIKON_TAG_UNKNOWN_0X00A8          = 0x00a8,
+	MNOTE_NIKON_TAG_TOTALPICTURES           = 0x00a7,	/* shuttercount */
+	MNOTE_NIKON_TAG_FLASHINFO          	= 0x00a8,
 	MNOTE_NIKON_TAG_OPTIMIZATION            = 0x00a9,
 	MNOTE_NIKON_TAG_SATURATION2             = 0x00aa,
 	MNOTE_NIKON_TAG_VARIPROGRAM             = 0x00ab,
+	MNOTE_NIKON_TAG_MULTIEXPOSURE           = 0x00b0,
+	MNOTE_NIKON_TAG_HIGHISONR           	= 0x00b1,
+	MNOTE_NIKON_TAG_TONINGEFFECT            = 0x00b3,
+	MNOTE_NIKON_TAG_POWERUPTIME             = 0x00b6,
+	MNOTE_NIKON_TAG_AFINFO2             	= 0x00b7,
+	MNOTE_NIKON_TAG_FILEINFO             	= 0x00b8,
+	MNOTE_NIKON_TAG_RETOUCHINFO            	= 0x00bb,
 	MNOTE_NIKON_TAG_CAPTUREEDITORDATA       = 0x0e01,
 	MNOTE_NIKON_TAG_CAPTUREEDITORVER	= 0x0e09,
 	MNOTE_NIKON_TAG_UNKNOWN_0X0E0E		= 0x0e0e,
@@ -115,7 +145,7 @@ enum _MnoteOlympusTag {
 	MNOTE_OLYMPUS_TAG_ONETOUCHWB		= 0x0302,
 	MNOTE_OLYMPUS_TAG_WHITEBALANCEBRACKET	= 0x0303,
 	MNOTE_OLYMPUS_TAG_WHITEBALANCEBIAS	= 0x0304,
-	MNOTE_OLYMPUS_TAG_UNKNOWN_5		= 0x0f00,
+	MNOTE_OLYMPUS_TAG_DATADUMP		= 0x0f00,
 	MNOTE_OLYMPUS_TAG_UNKNOWN_4		= 0x0f04,
 	MNOTE_OLYMPUS_TAG_SHUTTERSPEED		= 0x1000,
 	MNOTE_OLYMPUS_TAG_ISOVALUE		= 0x1001,
@@ -163,16 +193,23 @@ enum _MnoteOlympusTag {
 	MNOTE_OLYMPUS_TAG_LIGHTVALUECENTER	= 0x103d,
 	MNOTE_OLYMPUS_TAG_LIGHTVALUEPERIPHERY	= 0x103e,
 
+	/* Epson */
+	MNOTE_EPSON_TAG_IMAGE_WIDTH		= 0x020b,
+	MNOTE_EPSON_TAG_IMAGE_HEIGHT		= 0x020c,
+	MNOTE_EPSON_TAG_SOFTWARE		= 0x020d,
+
 	/* Sanyo */
 	MNOTE_SANYO_TAG_SEQUENTIALSHOT		= 0x020e,
 	MNOTE_SANYO_TAG_WIDERANGE		= 0x020f,
 	MNOTE_SANYO_TAG_COLORADJUSTMENTMODE	= 0x0210,
+	MNOTE_SANYO_TAG_FOCUSMODE		= 0x0212,
 	MNOTE_SANYO_TAG_QUICKSHOT		= 0x0213,
 	MNOTE_SANYO_TAG_SELFTIMER		= 0x0214,
 	MNOTE_SANYO_TAG_VOICEMEMO		= 0x0216,
 	MNOTE_SANYO_TAG_RECORDSHUTTERRELEASE	= 0x0217,
 	MNOTE_SANYO_TAG_FLICKERREDUCE		= 0x0218,
 	MNOTE_SANYO_TAG_OPTICALZOOM		= 0x0219,
+	MNOTE_SANYO_TAG_CCDSENSITIVITY		= 0x021a,
 	MNOTE_SANYO_TAG_DIGITALZOOM		= 0x021b,
 	MNOTE_SANYO_TAG_LIGHTSOURCESPECIAL	= 0x021d,
 	MNOTE_SANYO_TAG_RESAVED			= 0x021e,
@@ -186,9 +223,33 @@ typedef enum _MnoteOlympusTag MnoteOlympusTag;
 #define MNOTE_OLYMPUS_TAG_UNKNOWN_1	MNOTE_OLYMPUS_TAG_BWMODE
 #define MNOTE_OLYMPUS_TAG_UNKNOWN_2	MNOTE_OLYMPUS_TAG_FOCALPLANEDIAGONAL
 #define MNOTE_OLYMPUS_TAG_UNKNOWN_3	MNOTE_OLYMPUS_TAG_LENSDISTORTION
+#define MNOTE_OLYMPUS_TAG_UNKNOWN_5	MNOTE_OLYMPUS_TAG_DATADUMP
+#define MNOTE_NIKON_TAG_PREVIEWIMAGE	MNOTE_NIKON_TAG_PREVIEWIMAGE_IFD_POINTER
 
+/*! Return a textual name of the given tag within the Olympus-style MakerNote.
+ * The name is a short, unique, non-localized text string containing only
+ * US-ASCII alphanumeric characters.
+ *
+ * \param[in] tag Olympus-style MakerNote tag
+ * \return textual name of the tag, or NULL if the tag is unknown
+ */
 const char *mnote_olympus_tag_get_name        (MnoteOlympusTag tag);
+
+/*! Return a textual title of the given tag within the Olympus-style MakerNote.
+ * The title is a short, localized description of the tag.
+ *
+ * \param[in] tag Olympus-style MakerNote tag
+ * \return textual title of the tag, or NULL if the tag is unknown
+ */
 const char *mnote_olympus_tag_get_title       (MnoteOlympusTag tag);
+
+/*! Return a verbose textual description of the given tag within the
+ * Olympus-style MakerNote.
+ * The description is a verbose, localized description of the tag.
+ *
+ * \param[in] tag EXIF tag
+ * \return textual description of the tag, or NULL if the tag is unknown
+ */
 const char *mnote_olympus_tag_get_description (MnoteOlympusTag tag);
 
 #ifdef __cplusplus
